package dk.nsi.dds.projects.sxa.documentprovider.filter;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.util.Config;

public class HttpHeaderReaderFilter implements Filter {

    @Inject
    private HttpHeaderContext httpHeaderContext;

    @Inject
    private Config config;

	private List<String> headersToTransfer;

	@Override
	public void destroy() {
	}

	@Override
	public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain) throws IOException, ServletException {

		if (!(servletRequest instanceof HttpServletRequest)) {
            throw new IllegalStateException("servletrequest skal være http(s)!");
        }
		
        if(headersToTransfer == null) {
            throw new IllegalStateException(String.format("Property %s was not provided!", SXAConfigKeys.SXA_HEADERS));
        }

        // Extract the headers
        transferHttpHeadersFromRequest((HttpServletRequest) servletRequest);

        chain.doFilter(servletRequest, servletResponse);
	}
	
	
    public void transferHttpHeadersFromRequest(HttpServletRequest request) {

    	httpHeaderContext.reset(); // Prevent headers being reused due to thread reuse
    	
        for(String header : headersToTransfer) {
            String headerValue = request.getHeader(header);
            if(headerValue == null || header.isEmpty()) {
                continue;
            }

            httpHeaderContext.addHttpHeader(header, headerValue);
        }
    }


	@Override
	public void init(FilterConfig arg0) throws ServletException {
        // Get headers to transfer from config
        String headerProperty = config.getProperty(SXAConfigKeys.SXA_HEADERS);

        if(headerProperty != null) {
            headersToTransfer = Arrays.asList(headerProperty.split(","));
        }
	}

}
