package dk.nsi.dds.projects.sxa.documentprovider.filter;

import java.io.IOException;
import java.util.List;

import javax.enterprise.inject.Produces;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;

import org.jglue.cdiunit.CdiRunner;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.util.Config;

@RunWith(CdiRunner.class)
public class HttpHeaderReaderFilterTest {

	private static final String FOO = "foo";
	private static final String QUX = "qux";

	@InjectMocks
    private HttpHeaderReaderFilter subject;

    @Mock
    private HttpHeaderContext httpHeaderContext;
    
    @Produces
    @Mock
    private Config config;

	@Mock
	private FilterChain filterChain;

	private MockHttpServletRequest request = new MockHttpServletRequest();
	private MockHttpServletResponse response = new MockHttpServletResponse();

    
    @Before
    public void setup() throws IOException, ServletException {
    	
        Mockito.when(config.getProperty(SXAConfigKeys.SXA_HEADERS)).thenReturn(FOO);
    }

    @Test
    public void doFilter_canExtractFooHeaderWhenPresent() throws Exception {
        // Arrange
        request.addHeader(FOO, "bar");
        request.addHeader(FOO, "baz");
        subject.init(null);

        // Act
        subject.doFilter(request, response, filterChain);

        // Assert
        Mockito.verify(httpHeaderContext, Mockito.times(1)).addHttpHeader(Mockito.eq(FOO), Mockito.anyString());
    }

    @Test
    public void doFilter_resettingThreadValueOnFilter() throws Exception {
        // Arrange
        request.addHeader(FOO, "bar");
        subject.init(null);

        // Act
        subject.doFilter(request, response, filterChain);
    	request = new MockHttpServletRequest();
        request.addHeader(FOO, "bar2");
        subject.doFilter(request, response, filterChain);
        
        
        // Assert
        Mockito.verify(httpHeaderContext, Mockito.times(2)).addHttpHeader(Mockito.eq(FOO), Mockito.anyString());
        Mockito.verify(httpHeaderContext, Mockito.times(2)).reset();
    }


    @Test
    public void doFilter_allowsConfiguredHeadersToBeMissing() throws Exception {
        // Arrange
        subject.init(null);

        // Act
        subject.doFilter(request, response, filterChain);

        // Assert
        List<String> fooHeaders = httpHeaderContext.getHttpHeaders().get(FOO);
        Assert.assertNull(fooHeaders);
    }

    @Test
    public void doFilter_ignoresNonConfiguredHeaders() throws Exception {
        // Arrange
        request.addHeader(QUX, "bar");
        request.addHeader(QUX, "baz");
        subject.init(null);

        // Act
        subject.doFilter(request, response, filterChain);

        // Assert
        List<String> fooHeaders = httpHeaderContext.getHttpHeaders().get(FOO);
        Assert.assertNull(fooHeaders);
        List<String> quxHeaders = httpHeaderContext.getHttpHeaders().get(QUX);
        Assert.assertNull(quxHeaders);
    }
}
