package dk.nsi.dds.projects.sxa.common.security;

import java.io.IOException;

import javax.inject.Inject;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.xml.namespace.QName;

import org.apache.cxf.binding.soap.SoapFault;
import org.apache.log4j.Logger;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import dk.nsi.util.Config;
import dk.nsi.util.ConfigException;

public class AccessValidationFilter implements Filter {

	private static Logger logger = Logger.getLogger(AccessValidationFilter.class);

    @Inject
    private Config config;
    
    private String credentialHeaderName;
    private String serverCredential;
    
    
	@Override
	public void destroy() {
	}

	@Override
	public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
		
		validateAccess(servletRequest);
		filterChain.doFilter(servletRequest, servletResponse);
	}

	/**
	 * 
	 * @param servletRequest
	 * @return was validation enabled
	 */
	public boolean validateAccess(ServletRequest servletRequest) {
        if(credentialHeaderName != null && serverCredential != null) {
            checkCredential(servletRequest, serverCredential, credentialHeaderName);
            return true;
        } else if (credentialHeaderName != null || serverCredential != null) {
        	String message = "SXA was misconfigured, expected credentialHeader and serverCredential to be simultaneouly present or absent!";
        	logger.error(message);
            throwSoapFault(message);
        }
        return false;
	}
	
	public void checkCredential(ServletRequest servletRequest, String serverCredential, String credentialHeaderName) {
        String clientCredential = getCredentialHeaderValueFromRequest((HttpServletRequest) servletRequest, credentialHeaderName);
        if(!serverCredential.equals(clientCredential)) {
            throwSoapFault(String.format("Incorrect %s header provided!", credentialHeaderName));
        }
	}
	
    private String getCredentialHeaderValueFromRequest(HttpServletRequest request, String credentialHeaderName) {

        String headerValue = request.getHeader(credentialHeaderName);
        if(headerValue == null || headerValue.isEmpty()) {
            throwSoapFault(String.format("No header of type %s was present!", credentialHeaderName));
        }

        return headerValue;
    }
    
    private void throwSoapFault(String message) {
        throw new SoapFault(message, new QName("http://sxa", "bad_request"));
    }

	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		
        credentialHeaderName = getCredentialHeaderName();
        serverCredential = getServerCredential();
	}
	
    private String getCredentialHeaderName() {
        return getPropertyFromConfig(SXAConfigKeys.SXA_CREDENTIAL_HEADER);
    }

    private String getServerCredential() {
        return getPropertyFromConfig(SXAConfigKeys.SXA_CREDENTIAL);
    }

    private String getPropertyFromConfig(SXAConfigKeys configKey) {
        try {
            return config.getProperty(configKey);
        }
        catch(ConfigException e) {
            return null;
        }
    }
}
